DispatchFlasher = LibStub("AceAddon-3.0"):NewAddon("DispatchFlasher", "AceConsole-3.0", "AceEvent-3.0" );

--GLOBALS--
local _G = _G --Global enviroment table provided by WoW Api
local Buttons = {};
local DISPATCH_ID = 111240;
local DISPATCH_EXECUTE_RANGE = 35;
local SHADOWBURN_ID = 17877;
local SHADOWBURN_EXECUTE_RANGE = 20;
local PlayerClass = "";

local defaults = {
  global = {
    showSpellAlert = true,
	enabled = true
  }
}
  
-- Called when the addon is loaded
function DispatchFlasher:OnInitialize()

	-- Init DB
	self.db = LibStub("AceDB-3.0"):New("DispatchFlasherDB",defaults)

	local localizedClass, englishClass = UnitClass("player");
	PlayerClass = englishClass;
	if PlayerClass ~= "ROGUE" and PlayerClass ~="WARLOCK" then 
		self:Print("Not a Rogue or Warlock!! Addon will be disabled");
		DispatchFlasher:Disable();
		return;
	end

	-- Registers the slash commands handler
	self:RegisterChatCommand("dispatchflasher", "SlashCommandHandler", true);

	if self.db.global.enabled == false then
		DispatchFlasher:Disable();
		return;
	end
end

-- Called when the addon is enabled
function DispatchFlasher:OnEnable()

	-- Collects all the buttons of the UI in the global array Buttons
    Buttons = DispatchFlasher:GetAllButtons()

	--Registers the UNIT_HEALTH event that fires when a unit's health changes
	self:RegisterEvent("UNIT_HEALTH");

	--Registers the PLAYER_TARGET_CHANGED event. This event is fired whenever the player's target is changed, including when the target is lost. 
	self:RegisterEvent("PLAYER_TARGET_CHANGED");

	self:Print("To see a list of commands type /dispatchflasher help");
end

-- Called when the addon is disabled
function DispatchFlasher:OnDisable()
    
end

--UNIT_HEALTH event handler
--@param eventName Placeholder parameter by Ace. The name of the event
--@param arg1 the UnitID that triggered the event. e.x. "target","focus"
function DispatchFlasher:UNIT_HEALTH(eventName, arg1)
	if arg1 == "target" then
		DispatchFlasher:ShowOrHideFlasher();
	end
end

--PLAYER_TARGET_CHANGED event handler
--@param eventName Placeholder parameter by Ace. The name of the event
--@param arg1 The way the target was changed. e.x. Escape, Left click
function DispatchFlasher:PLAYER_TARGET_CHANGED(eventName, arg1)
	DispatchFlasher:ShowOrHideFlasher();
end

--Main function. Shows or hides the glow/spell alert accordingly
function DispatchFlasher:ShowOrHideFlasher()
	
	local health = UnitHealth("target");
	local maxHealth = UnitHealthMax("target"); 
	local percentage = (health/maxHealth) * 100;
	local executeRange = DispatchFlasher:GetExecuteRange();
		
	if percentage <= executeRange and not UnitIsDead("target") and UnitIsEnemy("player","target") then 
		-- Show Glow
		local foundButtons = DispatchFlasher:GetValidButtons(Buttons);

		for id,button in pairs(foundButtons) do
			ActionButton_ShowOverlayGlow(button);
		end

		--Show Spell Alert
		if self.db.global.showSpellAlert and table.getn(foundButtons) > 0  then
			DispatchFlasher:ShowSpellAlert();
		end

	else
		-- Hide Glow
		local foundButtons = DispatchFlasher:GetValidButtons(Buttons);
		for id,button in pairs(foundButtons) do
			ActionButton_HideOverlayGlow(button);
		end

		--Hide Spell Alert
		if self.db.global.showSpellAlert then
			DispatchFlasher:HideSpellAlert();
		end 
	end
end

--Gets the health percentage that execute procs depending on loged in class
function DispatchFlasher:GetExecuteRange()
	if PlayerClass == "ROGUE" then
		return DISPATCH_EXECUTE_RANGE;
	elseif PlayerClass == "WARLOCK" then
		return SHADOWBURN_EXECUTE_RANGE;
	end
end

--Gets the health percentage that execute procs depending on loged in class
function DispatchFlasher:ShowSpellAlert()
	if PlayerClass == "ROGUE" then
		SpellActivationOverlay_ShowOverlay(SpellActivationOverlayFrame, "DISPATCH_FLASHER_OVERLAY", "TEXTURES\\SPELLACTIVATIONOVERLAYS\\SUDDEN_DEATH.BLP", "RIGHT", 1, 255, 255, 255, false, true);
		SpellActivationOverlay_ShowOverlay(SpellActivationOverlayFrame, "DISPATCH_FLASHER_OVERLAY", "TEXTURES\\SPELLACTIVATIONOVERLAYS\\SUDDEN_DEATH.BLP", "LEFT", 1, 255, 255, 255, false, false);
	elseif PlayerClass == "WARLOCK" then
		SpellActivationOverlay_ShowOverlay(SpellActivationOverlayFrame, "DISPATCH_FLASHER_OVERLAY", "TEXTURES\\SPELLACTIVATIONOVERLAYS\\SUDDEN_DOOM.BLP", "RIGHT", 1, 255, 255, 255, false, true);
		SpellActivationOverlay_ShowOverlay(SpellActivationOverlayFrame, "DISPATCH_FLASHER_OVERLAY", "TEXTURES\\SPELLACTIVATIONOVERLAYS\\SUDDEN_DOOM.BLP", "LEFT", 1, 255, 255, 255, false, false);
	end
end 

--Gets the health percentage that execute procs depending on loged in class
function DispatchFlasher:HideSpellAlert()
		SpellActivationOverlay_HideOverlays(SpellActivationOverlayFrame, "DISPATCH_FLASHER_OVERLAY");
end 

--Finds the buttons that have the ability "Dispatch" or "Shadowburn"
--@param buttons The array of buttons to look into
function DispatchFlasher:GetValidButtons(buttons)
	
	local foundButtons = {};
	for name,button in pairs(buttons) do
		local LAB = LibStub("LibActionButton-1.0", true);
		local type,spellID,subType; 
		if(LAB) then --bartender
			spellID = button:GetSpellId();
		else		--blizzard
			type,spellID,subType  = GetActionInfo(button.action);
		end
		if(spellID == DISPATCH_ID or spellID == SHADOWBURN_ID) then
			table.insert(foundButtons,button);
		end
	end
	return foundButtons;
end

--Gets all the buttons of the UI. Currently Bartender and Blizzard UI are supported
function DispatchFlasher:GetAllButtons()
	
	local LAB = LibStub("LibActionButton-1.0", true);
	local buttons ={};
	if(LAB) then
		self:Print("Barteder Detected");
		for button in pairs(LAB:GetAllButtons()) do
			buttons[button:GetName()] = button;
		end
	else
		self:Print("BlizzardUI Detected");
		DispatchFlasher:GetBlizzardButtons("ActionButton",buttons);
		DispatchFlasher:GetBlizzardButtons("BonusActionButton",buttons);
		DispatchFlasher:GetBlizzardButtons("MultiBarRightButton",buttons);
		DispatchFlasher:GetBlizzardButtons("MultiBarLeftButton",buttons);
		DispatchFlasher:GetBlizzardButtons("MultiBarBottomRightButton",buttons);
		DispatchFlasher:GetBlizzardButtons("MultiBarBottomLeftButton",buttons);
	end
	return buttons;
end

-- Collects Blizzard's UI buttons and adds them to the given buttons array.
--@param barName The name of the blizzard bar to collect buttons from. e.x. "ActionButton", "MultiBarRightButton"
--@param buttonsArray The array to insert into the buttons found.
--@param count The number of buttons of this type. Default 12.
function DispatchFlasher:GetBlizzardButtons(buttonType, buttonsArray, count)
	for i=1,(count or 12) do
		if _G[buttonType .. i] ~= nil then
			buttonsArray[buttonType .. i] = _G[buttonType .. i];
		end
	end
end

-- Slash commands handler. 
--@param msg Contains the parameters passed after /dispatchflasher
--@param editbox Contains the chat window that was used to enter the command
function DispatchFlasher:SlashCommandHandler(msg, editbox)
	if msg == nil or msg == "" or msg == "help" then 
		self:Print("/dispatchflasher help  -- This info");		
		self:Print("/dispatchflasher status  -- Current status of the addon");	
		self:Print("/dispatchflasher spellalerts on  -- Turns spell alerts visual on");
		self:Print("/dispatchflasher spellalerts off  -- Turns spell alerts visual off");
		self:Print("/dispatchflasher enable -- Enables the addon");		
		self:Print("/dispatchflasher disable -- Disables the addon");
	elseif msg == "spellalerts on" then
		self.db.global.showSpellAlert = true;
		self:Print("Spell alerts: on");
	elseif msg == "spellalerts off" then
		self.db.global.showSpellAlert = false;
		self:Print("Spell alerts: off");
	elseif msg == "enable" then
		self.db.global.enabled = true;
		DispatchFlasher:Enable();
		self:Print("Enabled");
	elseif msg == "disable" then
		self.db.global.enabled = false;
		DispatchFlasher:Disable();
		self:Print("Disabled");
	elseif msg == "status" then
		if self.db.global.enabled then 
			self:Print("Enabled");
		else
			self:Print("Disabled");
		end
		if self.db.global.showSpellAlert then
			self:Print("Spell alerts: on");
		else
			self:Print("Spell alerts: off");
		end
	else
		self:Print("Unknown command. To see a list of commands type /dispatchflasher help");		
	end	
end